from .download import download_zip
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "assets.zip",down_path)
download_zip(xz_url + "func.zip",down_path)
download_zip(xz_url + "luts.zip",down_path)

import bpy
import os
from bpy.app.handlers import persistent
from bpy.types import PropertyGroup
from bpy.props import EnumProperty, StringProperty, PointerProperty, BoolProperty
import bpy.utils.previews

from .func.lut_functions import FinalLUT_Globals_LutFunction
from .func.globals import FinalLUT_Global_Functions, FinalLUT_Global_Variables
from .func.io_functions import FinalLUT_IO_Functions, refresh_previews, regenerate_textures, set_luts_directory
from .func.functions import FinalLUT_Compositor, ShowMessageBox, apply_lut_node, set_buffer_grouping, set_forced_settings
from .func.lut_classes import LUT_Object

from .translation.translate import GetTranslationDict


bl_info = {
    "name": "Final LUT",
    "author": "Alexandre Labedade, Kevin Deguisne, Antoine Bagattini",
    "version": (1, 0, 5),
    "blender": (4, 1, 0),
    "location": "3D View > Final LUT | Compositor > Final LUT",
    "description": "Apply colors look up tables (LUT) in the compositor",
    "category": "Compositing",
    "warning": "",
    "doc_url": "https://blendermarket.com/products/finallut/docs",
    "tracker_url":""
}

######################### GLOBAL FUNCTIONS #########################

def reset_luts_directory():
    set_luts_directory(os.path.join(os.path.dirname(__file__), "luts"))
    FinalLUT_Global_Functions.clear_previews()
    FinalLUT_Global_Functions.refresh_previews()
    
def setup_preview_collections():
    
    if not FinalLUT_Global_Variables.preview_collections:
        previews = {}
        
        def create_new_previews_collection():
            pcoll = bpy.utils.previews.new()
            pcoll.directory = ""
            pcoll.first = ""
            pcoll.neutral = ""
            pcoll.filtered_previews = ()
            pcoll.need_update = True
            return pcoll
            
        previews["default"] = create_new_previews_collection()
        previews["viewport"] = create_new_previews_collection()
        previews["render"] = create_new_previews_collection()
        
        
        FinalLUT_Global_Variables.preview_collections = previews

def enum_previews_from_directory_items(self, context):

    if FinalLUT_Global_Variables.reloading_thumbnail:
        return [('NONE', 'NONE', '', '', FinalLUT_Global_Variables.last_index)]
    
    if not FinalLUT_Global_Variables.preview_collections:
        return [('NONE', 'NONE', '', '', FinalLUT_Global_Variables.last_index)]
    
    
    if FinalLUT_Global_Variables.refresh_previews:
        FinalLUT_Global_Variables.refresh_previews = False
        
        result = refresh_previews()
    else:
        final_lut_props = FinalLUT_Global_Variables.get_props()
        pcoll = FinalLUT_Global_Variables.preview_collections.get(final_lut_props.lut_previews_modes)

        result = pcoll.filtered_previews
    
    return result

def refresh_in_use():
    
    final_lut_props = FinalLUT_Global_Variables.get_props()
    pcoll = FinalLUT_Global_Variables.preview_collections.get(final_lut_props.lut_previews_modes)
    
    keep_luts = FinalLUT_Global_Functions.get_used_luts()
    
    for i,item in enumerate(pcoll.filtered_previews):
        name = item[0]
        display_name = item[1]
        
        prefix = ''
        if name in keep_luts:
            prefix = prefix + FinalLUT_Global_Variables.in_use_label
            
        if name in FinalLUT_Global_Variables.favorites:
           prefix = prefix + FinalLUT_Global_Variables.favorites_label

        display_name = display_name.replace(FinalLUT_Global_Variables.in_use_label,'')
        display_name = display_name.replace(FinalLUT_Global_Variables.favorites_label,'')
        display_name = f"{prefix} {display_name.strip()}".strip()
        pcoll.filtered_previews[i] = (item[0],display_name,item[2],item[3],item[4])

@persistent
def render_pre_handler(scene):
    set_buffer_grouping()
    FinalLUT_Global_Variables.click_detection_enabled = False

@persistent
def composite_pre_handler(scene):
    #print("COMPO PRE")
    FinalLUT_Compositor.busy = True

@persistent
def composite_post_handler(scene):
    #print("COMPO POST")
    FinalLUT_Compositor.busy = False

@persistent
def load_post_handler(filepath):
    FinalLUT_Global_Variables.refresh_previews = True
    FinalLUT_Global_Variables.new_file_loaded = True
    FinalLUT_Global_Variables.first_RT_display_done = False
    FinalLUT_Global_Functions.upgrade_to_current_version()

@persistent
def save_pre_handler(filepath):
    set_buffer_grouping()
    
    # Remove unused images
    for image in bpy.data.images:
        if image.name.startswith('.LUT_') and image.packed_file and image.users == 0:
            bpy.data.images.remove(image)
            
    # Pack all LUT textures used in this file     
    LUT_NodeGroup = FinalLUT_Global_Functions.get_finallut_group()
    for scene in bpy.data.scenes:
        if scene.node_tree and scene.node_tree.nodes:
            for node in scene.node_tree.nodes:
                if node.type == 'GROUP' and node.node_tree == LUT_NodeGroup:
                    if node.inputs['LUT'].links:
                        from_node = node.inputs['LUT'].links[0].from_node
                        while from_node.type != 'IMAGE' and from_node.inputs and from_node.inputs[0].links:
                            from_node = from_node.inputs[0].links[0].from_node
                        
                        if from_node.type == 'IMAGE' and from_node.image:
                            if FinalLUT_Global_Variables.script_path in from_node.image.filepath or FinalLUT_Global_Variables.presets_path in from_node.image.filepath:
                                from_node.image.pack()

def preview_mode_change(prop,context):
    FinalLUT_Global_Variables.refresh_previews = True

def previewChange(prop,context):
    if FinalLUT_Global_Variables.prevent_lut_refresh:
        return
       
    apply_lut_node(prop.lut_previews)
    
    #FinalLUT_Global_Variables.refresh_previews = True
    #result = enum_previews_from_directory_items(None,context)
    #FinalLUT_Global_Variables.refresh_previews = False
    
    toggle_filter(None,context)


def get_directory_list(self, context):
    path = FinalLUT_IO_Functions.get_luts_directory()
    if path:
        result =  ((name, name, 'description', 'FILE_FOLDER', i) for i,name in enumerate(os.listdir(path)) if os.path.isdir(os.path.join(path, name)))   
        return result
    else:
        return ()

def toggle_all_nodes(props,context):
    
    if context.scene.node_tree:
        nodes = context.scene.node_tree.nodes
        #sel = [x for x in nodes if x.type == 'GROUP' and x.node_tree.name == FinalLUT_Global_Variables.nodegroup_name]
        sel = [x for x in nodes if x.type == 'GROUP' and FinalLUT_Global_Functions.match_nodegroup_name(x.node_tree.name)]
        for node in sel:
            mix = FinalLUT_Global_Functions.get_descendant_node_by_type('MIX_RGB',node)
            if mix:
                mix.mute = not props.state
            else:
                node.mute = not props.state
                
                exposure = FinalLUT_Global_Functions.get_ascendant_node_by_type('EXPOSURE',node)
                if exposure:
                    exposure.mute = not props.state
                    
                colorspace = FinalLUT_Global_Functions.get_ascendant_node_by_type('CONVERT_COLORSPACE',node)
                if colorspace:
                    colorspace.mute = not props.state

                mix_node = node.node_tree.nodes.get('LUT Mix')
                if mix_node:
                    mix_node.mute = not props.state
              
        
def select_category(self,context):
    FinalLUT_Global_Variables.prevent_lut_refresh = True
    
    FinalLUT_Global_Variables.refresh_previews = True
    FinalLUT_Global_Functions.clear_previews()
  
    FinalLUT_Global_Variables.prevent_lut_refresh = False
    return None

def toggle_filter(self,context):
    FinalLUT_Global_Variables.prevent_lut_refresh = True
    
    final_lut_props = FinalLUT_Global_Variables.get_props()
    current_preview_name = str(final_lut_props.lut_previews)
    
    FinalLUT_Global_Variables.refresh_previews = True
    result = enum_previews_from_directory_items(self,context)
    
    found = False
    for r in result:
        if r[0] == current_preview_name:
            found = True
            
    if found:
        final_lut_props.lut_previews = current_preview_name
    elif final_lut_props.lut_previews!="":
        final_lut_props.lut_previews = final_lut_props.lut_previews
    elif result:
        final_lut_props.lut_previews = result[0][0]
        
    FinalLUT_Global_Variables.prevent_lut_refresh = False
    
    return None

def get_attributes(self,context,name):
    list = LUT_Object.attributes.get(name)
    result = []
    if list:
        for item in list:
            result.append((item,item,''))
    return result

def select_attribute(self,context,name):

    FinalLUT_Global_Variables.prevent_lut_refresh = True
    
    final_lut_props = FinalLUT_Global_Variables.get_props()
    current_preview_name = final_lut_props.lut_previews
    
    FinalLUT_Global_Variables.refresh_previews = True
    enum_previews_from_directory_items(self,context)
    #FinalLUT_Global_Functions.select_first_preview()
    
    if FinalLUT_Global_Variables.focus_after_import:
        if FinalLUT_Global_Variables.focus_after_import in final_lut_props.lut_previews:
            current_preview_name = FinalLUT_Global_Variables.focus_after_import
    
    if current_preview_name in final_lut_props.lut_previews:
       final_lut_props.lut_previews = current_preview_name
    
    FinalLUT_Global_Variables.prevent_lut_refresh = False
    return None

def change_shading_visibility(props,context):
    if props.show_vis_options:
        if context.space_data.shading.type in ['MATERIAL','RENDERED']:
            props.shading_mode = context.space_data.shading.type
        else:
            context.space_data.shading.type = props.shading_mode

        if props.shading_context != 'OFF':
            context.space_data.shading.use_compositor = props.shading_context
        
    else:
        context.space_data.shading.use_compositor = 'DISABLED'

def change_shading_mode(props,context):
    
    if context.space_data and not context.space_data.shading.type in ('MATERIAL','RENDERED'):
        props.shading_context = 'OFF'
    
    if props.show_vis_options:
        if props.shading_context != 'OFF':
            context.space_data.shading.use_compositor = props.shading_context
            
        context.space_data.shading.type = props.shading_mode
    else:
        context.space_data.shading.use_compositor = 'DISABLED'
    pass

def change_luts_directory(props,context):
    regenerate_textures()

def change_force_standard_viewtransform(props,context):
    FinalLUT_Global_Variables.force_standard_viewtransform = props.force_standard_viewtransform

def get_preview(props):
    
    active_name = FinalLUT_Compositor.get_active_lut_name()
    
    if active_name:
        pcoll = FinalLUT_Global_Variables.preview_collections.get(props.lut_previews_modes)
        if pcoll.filtered_previews:
            for item in pcoll.filtered_previews:
                if item[0] == active_name:
                    props['lut_previews'] = item[4]
                    FinalLUT_Global_Variables.last_index = item[4]
                    return item[4]
    return props.get('lut_previews',0)

def set_preview(props,value):

    pcoll = FinalLUT_Global_Variables.preview_collections.get(props.lut_previews_modes)
    if pcoll.filtered_previews:
        total = len(pcoll.filtered_previews)
        if value > -1 and value < total:
            props['lut_previews'] = value
            FinalLUT_Global_Variables.last_index = value
        else:
            print("WRONG INDEX")

def get_sort_items(self, context):
    items = [('name','Name','')]
    for key in LUT_Object.attributes.keys():
        items.append((key,key.capitalize(),''))
    return items

def apply_sort_by(self, context):
    refresh_previews()

class FinalLUT_Properties(PropertyGroup):
    
    lut_previews: EnumProperty(
        name = "LUT Previews",
        description = "List of LUT Thumbnails",
        items = enum_previews_from_directory_items,
        update = previewChange,
        set=set_preview,
        get=get_preview
    ) # type: ignore
    
    lut_previews_modes: EnumProperty(
        name = "Previews Mode",
        description = "Preview display source",
        items = (
            ('default','Default','Use the default thumbnail image','SEQ_PREVIEW',1),
            ('viewport','Viewport thumbnails','Use the last EEVEE viewport capture as thumbnail source','SHADING_TEXTURE',2),
            ('render','Render thumbnails','Use the current render result slot as thumbnail source, falls back to viewport if none is available','SHADING_RENDERED',3),
            ),
        update = preview_mode_change
    ) # type: ignore
    
    selected_category: bpy.props.EnumProperty(
        items=get_directory_list,
        name="Select a Category",
        description="Select a Category",
        update=select_category
    ) # type: ignore
    
    state: BoolProperty(
        name="Toggle Rendering",
        description="Toggle all Final LUT nodes in the compositor",
        default = True,
        update=toggle_all_nodes
        ) # type: ignore
    
    show_filters: BoolProperty(
        name="Show Filters",
        description="",
        default = False,
        update=toggle_filter
        ) # type: ignore
    
    show_details: BoolProperty(
        name="Show LUT details",
        description="",
        default = False
        ) # type: ignore

    filter_by_name: StringProperty(
        name="Filter by name",
        description="Search pattern in LUTs names",
        default = '',
        update=toggle_filter
    ) # type: ignore
    
    filter_by_favorites: BoolProperty(
        name="Filter favorites",
        description="Filter favorites",
        default = False,
        update=toggle_filter
        ) # type: ignore

    filter_by_gamma: BoolProperty(
        name="Filter list by GAMMA",
        description="",
        default = False,
        update=toggle_filter
        ) # type: ignore
    
    selected_gamma: bpy.props.EnumProperty(
        items=lambda self, context: get_attributes(self, context, "gamma"),
        name="GAMMA",
        description="",
        update=lambda self, context: select_attribute(self, context, "gamma"), 
    ) # type: ignore
    
    filter_by_color: BoolProperty(
        name="Filter list by COLOR",
        description="",
        default = False,
        update=toggle_filter
        ) # type: ignore
    
    selected_color: bpy.props.EnumProperty(
        items=lambda self, context: get_attributes(self, context, "color"),
        name="COLOR",
        description="",
        update=lambda self, context: select_attribute(self, context, "color")
    ) # type: ignore
    
    filter_by_key: BoolProperty(
        name="Filter list by KEY",
        description="",
        default = False,
        update=toggle_filter
        ) # type: ignore
    
    selected_key: bpy.props.EnumProperty(
        items=lambda self, context: get_attributes(self, context, "key"),
        name="KEY",
        description="",
        update=lambda self, context: select_attribute(self, context, "key")
    ) # type: ignore
    
    filter_by_style: BoolProperty(
        name="Filter list by STYLE",
        description="",
        default = False,
        update=toggle_filter
        ) # type: ignore
    
    selected_style: bpy.props.EnumProperty(
        items=lambda self, context: get_attributes(self, context, "style"),
        name="STYLE",
        description="",
        update=lambda self, context: select_attribute(self, context, "style")
    ) # type: ignore
    
    sort_by: bpy.props.EnumProperty(
        items=get_sort_items,
        name="Sort by",
        description="Sort by",
        update=apply_sort_by
        ) # type: ignore
    

class FinalLUT_Preferences(bpy.types.AddonPreferences):
    bl_idname = __package__
    
    luts_directory: StringProperty(
        name="LUTs Directory",
        subtype='DIR_PATH',
        default=os.path.join(os.path.dirname(__file__), "luts"),
        update=change_luts_directory
    ) # type: ignore
    
    thumbnails_exposure: BoolProperty(
        name="Apply Exposure to thumbnails",
        default=True
    ) # type: ignore
    
    force_standard_viewtransform: BoolProperty(
        name="Force the view transform to 'Standard' setting",
        description="This setting ensures the correct colors output, disable it if you want to manually set the View Transform in the Color Management panel.",
        default=True,
        update=change_force_standard_viewtransform
    ) # type: ignore
    
    default_realtime_compositor_mode: bpy.props.EnumProperty(
        items=(
            ('DISABLED','Disabled','','',1),
            ('CAMERA','Camera','','',2),
            ('ALWAYS','Always','','',3),
            ),
        name="Default RT Compositor display",
        description="Set the default realtime compositor display mode in the 3D viewport",
        default='ALWAYS'
    ) # type: ignore
    
    
    def draw(self, context):
        layout = self.layout
        
        # if FinalLUT_Global_Variables.current_version != (1,0,5):
        #     box = layout.box()
        #     box.label(text='RELOAD BLENDER TO VALIDATE THE ADDON UPDATE',icon='ERROR')
            # return
        
        if FinalLUT_Global_Variables.import_progress == 0:
            box = layout.box()
            box.label(text="Default Realtime Compositor display mode:")
            row = box.row()
            row.prop(self,'default_realtime_compositor_mode',expand=True)

            layout.separator()
            
            box = layout.box()
            box.prop(self,'thumbnails_exposure')
            box.separator()            
            box.prop(self,'force_standard_viewtransform',icon="ERROR")
            
            layout.separator()
            
            box = layout.box()
            
            row = box.row(align=True)
            row.label(text="LUTs directory:")
            row.operator("final_lut.reset_directory", text="Reset path and reload")
            
            row = box.row(align=True)
            row.prop(self, "luts_directory",text="")

        layout.separator()
        box = layout.box()
        row = box.row()
        row.alignment = 'CENTER'
        row.scale_y = 1.5
        
        if FinalLUT_Global_Variables.import_progress > 0:
            row.label(text="Processing LUT files...")
            row = box.row()
            row.alignment = 'CENTER'
            row.label(text=FinalLUT_Global_Variables.import_current_name)
            row = box.row()
            row.alignment = 'CENTER'
            row.label(text=f"{round(FinalLUT_Global_Variables.import_progress*100,1)}%")
        else:
            row.operator("final_lut.refresh", text="Reload all LUT data files (this may take some seconds)")


######################### REGISTER ########################
        
classes = [
    FinalLUT_Properties,
    FinalLUT_Preferences
]

for script in [
        'panel','io_functions'
    ]:
    exec(f"from .func import {script}")
    exec(f"for cls in {script}.classes: classes.append(cls)")




def register():
   
    FinalLUT_Global_Variables.script_path = os.path.dirname(__file__)
    FinalLUT_Global_Variables.addon_internal_name = __package__
    
    FinalLUT_Global_Variables.presets_path = os.path.abspath(os.path.join(FinalLUT_Global_Variables.script_path,'../../presets/FinalLUT'))
    if not os.path.isdir(FinalLUT_Global_Variables.presets_path):
        os.makedirs(FinalLUT_Global_Variables.presets_path)
    
    for cls in classes:
        try:
            bpy.utils.register_class(cls)
        except:
            continue

    bpy.types.Scene.final_lut_props = PointerProperty(type=FinalLUT_Properties)

    bpy.app.handlers.render_pre.append(render_pre_handler)
    
    bpy.app.handlers.composite_pre.append(composite_pre_handler)
    bpy.app.handlers.composite_post.append(composite_post_handler)
    bpy.app.handlers.load_post.append(load_post_handler)
    bpy.app.handlers.save_pre.append(save_pre_handler)
    
    try:
        FinalLUT_Globals_LutFunction.clear_pixels()
        FinalLUT_Global_Variables.delayed_reload_started = False
        FinalLUT_Global_Variables.refresh_previews = True
        FinalLUT_Global_Variables.preview_mode = 'default'

        # FinalLUT_Global_Functions.init_versioned_data()
        
        setup_preview_collections()
        
        FinalLUT_Global_Functions.add_neutral_lut()
            
        bpy.app.timers.register(lambda : FinalLUT_IO_Functions.load_favorites(), first_interval=0.15)
        bpy.app.timers.register(lambda : set_forced_settings(), first_interval=0.2)
    except:
        # print("Please restart Blender to validate the Final LUT update.")
        # ShowMessageBox("Please restart Blender to validate the Final LUT update.")
        pass
    try:
        bpy.app.translations.register(__package__, GetTranslationDict())
    except Exception as e: print(e)
    
   
def unregister():
    try:
        bpy.app.handlers.render_pre.remove(render_pre_handler)
        
        bpy.app.handlers.composite_pre.remove(composite_pre_handler)
        bpy.app.handlers.composite_post.remove(composite_post_handler)
        bpy.app.handlers.load_post.remove(load_post_handler)
        bpy.app.handlers.save_pre.remove(save_pre_handler)
    except:
        print("unregister error")
    
    del bpy.types.Scene.final_lut_props
        
    for cls in classes:
        bpy.utils.unregister_class(cls)
    
    FinalLUT_Global_Functions.clear_data()
    try:
        bpy.app.translations.unregister(__package__)
    except Exception as e: print(e)
    
if __name__ == "__main__":
    register()
    